#define _XTAL_FREQ 32000000

#include <xc.h>
#include <stdio.h>
#include "Software_UART.h"

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = ON       // PLL Enable (4x PLL enabled)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

int main(void){

    //         PIC    MAX6675
    // SCK1(10[11]) - SCK
    //   SDI1(7[8]) - SO
    //LATB7(13[14]) - CS

    //         PIC    FT232RL
    //LATB5(11[12]) - RX
    //LATB6(12[13]) - TX
    //See Software_UART.h

    //Pin (PDIP[SSOP])

    OSCCONbits.IRCF = 0b1110;//8MHz x 4

    //SPI configuration
    ANSELBbits.ANSB4 = 0;//
    ANSELBbits.ANSB1 = 0;//
    ANSELBbits.ANSB2 = 0;//
    ANSELBbits.ANSB7 = 0;//
    TRISBbits.TRISB4 = 0;//SCK
    TRISBbits.TRISB1 = 1;//SDI
    TRISBbits.TRISB2 = 0;//SDO
    TRISBbits.TRISB7 = 0;//CS

    SSP1CON1bits.CKP = 0;
    SSP1STATbits.CKE = 1;
    SSP1CON1bits.SSPM = 0b0000;//SPI Master mode, clock = FOSC/4 = 4MHz
    SSP1STATbits.SMP = 0;
    SSP1CON1bits.SSPEN = 1;//Synchronous Serial Port Enabled

    unsigned char i;
    unsigned int a = 0;
    char b[5];

    InitSoftUART();//Intialize Soft UART

    LATBbits.LATB7 = 1;//CS of MAX6675

    while(1) {
        __delay_ms(500);
        LATBbits.LATB7 = 0;//CS of MAX6675 to Read Data
        __delay_ms(1);
        SSP1BUF = 0x01;
        while(!SSP1STATbits.BF);//Wait for SSP1BUF will be full
        a = SSP1BUF;//Read the First 8 bits
        a <<= 8;//Shift 8 bits
        SSP1BUF = 0x01;
        while(!SSP1STATbits.BF);//Wait for SSP1BUF will be full
        a |= SSP1BUF;//Read the Second 8 bits
        //Thermocouple Input, Device ID and State = Least 3 bits
        //Least 3 bits and devide by 4 (1 bit as 0.25 degree)
        a >>= 5;
        sprintf(b, "%4d", a);
        for(i=0; i<4; i++){
            UART_Transmit(b[i]);
        }
        UART_Transmit(0x0D);
        UART_Transmit(0x0A);
        LATBbits.LATB7 = 1;//CS of MAX6675 to Initiate a New Conversion Process
        __delay_ms(500);
    }
}